#pragma once

#include <atltime.h>
#include <list>

static const size_t cMaxSegmentName = 256;

#pragma pack(push)
#pragma pack(1)

class CSCTE104SingleOperationMessage
{
public:
	CSCTE104SingleOperationMessage() {
		opID				=	0;
		messageSize			=	0;
		result				=	0xFFFF;
		result_extension	=	0xFFFF;
		protocol_version	=	0x00;
		AS_index			=	0;
		message_number		=	0;
		DPI_PID_index		=	0;
	}
protected:
	// SCTE104 packet
	USHORT		opID;
	USHORT		messageSize;
	USHORT		result;
	USHORT		result_extension;
	UCHAR		protocol_version;
	UCHAR		AS_index;
	UCHAR		message_number;
	USHORT		DPI_PID_index;
};

class CAliveRequest : public CSCTE104SingleOperationMessage
{
public:
	CAliveRequest() {
		opID				=	htons(0x0003);
		messageSize			=	htons(USHORT(sizeof(*this)));
	}
	void Fill(BYTE nMsgID, DWORD64 UTCTime) {
		message_number		=	nMsgID;
		CTime		zt		=	CTime(1980, 1, 6, 0, 0, 0).GetTime();
		SYSTEMTIME	zst;
		CFileTime	st;
		zt.GetAsSystemTime(zst);
		SystemTimeToFileTime(&zst, &st);
		//
		ULONGLONG	t		=	(UTCTime - st.GetTime()) / 10;
		seconds				=	htonl(ULONG(t / 1000000));
		microseconds		=	htonl(ULONG(t % 1000000));
	}
protected:
	ULONG	seconds;
	ULONG	microseconds;
};

class SDIPacketHeaderPlus
{
public:
	SDIPacketHeaderPlus(UCHAR data_size) {
		header[0]	= 0x00;				// fixed SDI packet header: 0x00 + 0xFF + 0xFF
		header[1]	= 0xff;
		header[2]	= 0xff;
		did			= 0x41;				// Data ID for SCTE-104 data
		sdid		= 0x07;				// Secondary Data ID for SCTE-104 data
		length		= 1 + data_size;	// Payload flags + SCTE-104 message data
		flags		= 0x08;				// begin and end of data (0x08=???, 0x04=not_end, 0x02=not_start)
	}
protected:
	// SDI packet header
	UCHAR		header[3];
	UCHAR		did;
	UCHAR		sdid;
	UCHAR		length;
	// additional flags for continuous data = Payload Flags
	UCHAR		flags;
	// ... SCTE-104 message data
	// last UCHAR of the packet is CheckSum (9-bit + control bit)
};

#pragma pack(pop)
